"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearUISelection = clearUISelection;
exports.getUISelection = getUISelection;
exports.prepareSelectionInterceptor = prepareSelectionInterceptor;
exports.setUISelection = setUISelection;

var _interceptor = require("./interceptor");

const UISelection = Symbol('Displayed selection in UI');

function prepareSelectionInterceptor(element) {
  (0, _interceptor.prepareInterceptor)(element, 'setSelectionRange', function interceptorImpl(start, end, direction = 'none') {
    const isUI = start && typeof start === 'object' && start[UISelection];
    this[UISelection] = isUI ? {
      start: start.valueOf(),
      end: Number(end)
    } : undefined;
    return {
      realArgs: [Number(start), end, direction]
    };
  });
  (0, _interceptor.prepareInterceptor)(element, 'selectionStart', function interceptorImpl(v) {
    this[UISelection] = undefined;
    return {
      realArgs: v
    };
  });
  (0, _interceptor.prepareInterceptor)(element, 'selectionEnd', function interceptorImpl(v) {
    this[UISelection] = undefined;
    return {
      realArgs: v
    };
  });
}

function setUISelection(element, start, end) {
  element[UISelection] = {
    start,
    end
  };

  if (element.selectionStart === start && element.selectionEnd === end) {
    return;
  } // eslint-disable-next-line no-new-wrappers


  const startObj = new Number(start);
  startObj[UISelection] = UISelection;

  try {
    element.setSelectionRange(startObj, end);
  } catch {// DOMException for invalid state is expected when calling this
    // on an element without support for setSelectionRange
  }
}

function getUISelection(element) {
  const ui = element[UISelection];
  return ui === undefined ? {
    selectionStart: element.selectionStart,
    selectionEnd: element.selectionEnd
  } : {
    selectionStart: ui.start,
    selectionEnd: ui.end
  };
}

function clearUISelection(element) {
  element[UISelection] = undefined;
}