"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSelectionRange = getSelectionRange;
exports.setSelectionRange = setSelectionRange;

var _isElementType = require("../misc/isElementType");

var _document = require("../../document");

function getSelectionRange(element) {
  if ((0, _isElementType.isElementType)(element, ['input', 'textarea'])) {
    return (0, _document.getUISelection)(element);
  }

  const selection = element.ownerDocument.getSelection(); // there should be no editing if the focusNode is outside of element
  // TODO: properly handle selection ranges

  if (selection != null && selection.rangeCount && element.contains(selection.focusNode)) {
    const range = selection.getRangeAt(0);
    return {
      selectionStart: range.startOffset,
      selectionEnd: range.endOffset
    };
  } else {
    return {
      selectionStart: null,
      selectionEnd: null
    };
  }
}

function setSelectionRange(element, newSelectionStart, newSelectionEnd) {
  if ((0, _isElementType.isElementType)(element, ['input', 'textarea'])) {
    return (0, _document.setUISelection)(element, newSelectionStart, newSelectionEnd);
  }

  const {
    selectionStart,
    selectionEnd
  } = getSelectionRange(element); // Prevent unnecessary select events
  // istanbul ignore next

  if (selectionStart === newSelectionStart && selectionEnd === newSelectionEnd) {
    return;
  }

  const range = element.ownerDocument.createRange();
  range.selectNodeContents(element); // istanbul ignore else

  if (element.firstChild) {
    range.setStart(element.firstChild, newSelectionStart);
    range.setEnd(element.firstChild, newSelectionEnd);
  }

  const selection = element.ownerDocument.getSelection(); // istanbul ignore else

  if (selection) {
    selection.removeAllRanges();
    selection.addRange(range);
  }
}