"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FakePointerEvent = exports.FakeMouseEvent = void 0;

// See : https://github.com/testing-library/react-testing-library/issues/268
function assignProps(obj, props) {
  for (const [key, value] of Object.entries(props)) {
    Object.defineProperty(obj, key, {
      get: () => value
    });
  }
}

function assignPositionInit(obj, {
  x,
  y,
  clientX,
  clientY,
  offsetX,
  offsetY,
  pageX,
  pageY,
  screenX,
  screenY
}) {
  var _ref, _ref2, _ref3, _ref4;

  assignProps(obj, {
    /* istanbul ignore start */
    x: (_ref = x != null ? x : clientX) != null ? _ref : 0,
    y: (_ref2 = y != null ? y : clientY) != null ? _ref2 : 0,
    clientX: (_ref3 = x != null ? x : clientX) != null ? _ref3 : 0,
    clientY: (_ref4 = y != null ? y : clientY) != null ? _ref4 : 0,
    offsetX: offsetX != null ? offsetX : 0,
    offsetY: offsetY != null ? offsetY : 0,
    pageX: pageX != null ? pageX : 0,
    pageY: pageY != null ? pageY : 0,
    screenX: screenX != null ? screenX : 0,
    screenY: screenY != null ? screenY : 0
    /* istanbul ignore end */

  });
}

function assignPointerInit(obj, {
  isPrimary,
  pointerId,
  pointerType
}) {
  assignProps(obj, {
    isPrimary,
    pointerId,
    pointerType
  });
}

const notBubbling = ['mouseenter', 'mouseleave', 'pointerenter', 'pointerleave'];

function getInitDefaults(type, init) {
  return {
    bubbles: !notBubbling.includes(type),
    cancelable: true,
    composed: true,
    ...init
  };
}

class FakeMouseEvent extends MouseEvent {
  constructor(type, init) {
    super(type, getInitDefaults(type, init));
    assignPositionInit(this, init);
  }

} // Should extend PointerEvent, but... https://github.com/jsdom/jsdom/issues/2527


exports.FakeMouseEvent = FakeMouseEvent;

class FakePointerEvent extends MouseEvent {
  constructor(type, init) {
    super(type, getInitDefaults(type, init));
    assignPositionInit(this, init);
    assignPointerInit(this, init);
  }

}

exports.FakePointerEvent = FakePointerEvent;