"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.firePointerEvent = firePointerEvent;

var _dom = require("@testing-library/dom");

var _fakeEvent = require("./fakeEvent");

var _mouseButtons = require("./mouseButtons");

function firePointerEvent(target, type, {
  pointerState,
  keyboardState,
  pointerType,
  button,
  coords,
  pointerId,
  isPrimary,
  clickCount
}) {
  const Event = type === 'click' || type.startsWith('mouse') ? _fakeEvent.FakeMouseEvent : _fakeEvent.FakePointerEvent;
  const init = { ...coords,
    altKey: keyboardState.modifiers.alt,
    ctrlKey: keyboardState.modifiers.ctrl,
    metaKey: keyboardState.modifiers.meta,
    shiftKey: keyboardState.modifiers.shift
  };

  if (Event === _fakeEvent.FakePointerEvent || type === 'click') {
    init.pointerId = pointerId;
    init.pointerType = pointerType;
  }

  if (['pointerdown', 'pointerup'].includes(type)) {
    init.isPrimary = isPrimary;
  }

  if (['mousedown', 'mouseup', 'pointerdown', 'pointerup', 'click'].includes(type)) {
    init.button = (0, _mouseButtons.getMouseButton)(button != null ? button : 0);
    init.buttons = (0, _mouseButtons.getMouseButtons)(...pointerState.pressed.filter(p => p.keyDef.pointerType === pointerType).map(p => {
      var _p$keyDef$button;

      return (_p$keyDef$button = p.keyDef.button) != null ? _p$keyDef$button : 0;
    }));
  }

  if (['mousedown', 'mouseup', 'click'].includes(type)) {
    init.detail = clickCount;
  }

  return (0, _dom.fireEvent)(target, new Event(type, init));
}